"""CLI submodules for gentroutils package."""

from gentroutils.commands.update_gwas_curation_metadata import (
    update_gwas_curation_metadata_command,
)

__all__ = ["update_gwas_curation_metadata_command"]
